/*
   mbrot.c - Program to create a picture of a portion of the Mandelbrot set
	     for the equation Z = Z + C

		   Program by Brian Irvine
		   Version 1.0	   1/1/86

	     Allows choosing an area of the complex plane to be
	     displayed by specifying the center of the picture
	     and the width of the area to be displayed, then
	     calculates points for the chosen domain and prints
	     them on an Epson RX, FX or MX printer with 960x576
	     dot resolution.

   This is an implementation of the program Mandelzoom described by
   A. K. Dewdney in the August 1985 edition of Scientific American.

   Further material is available in a beautifully produced book titled
   "Frontiers of Chaos" ("Schonheit Im Chaos") by H. O. Peitgen and
   P. H. Richter of the Forschungsgruppe Komplexe Dynamik, University of
   Bremen, now available in bookstores.

*/

#include   "stdio.h"
#include   "math.h"


/*----- Physical attributes of IBM color screen ----------------------------*/

#define    MAXCOL  960	   /* 960 dots available across */
#define    MAXROW  72	   /* 72 lines @ 9 lpi = 8 inches high */
#define    ASPECT  1.000   /* Area aspect ratio ( h/w ) */

/*--------------------------------------------------------------------------*/

#define    MAXPTS  7680    /* 8 rows of dots on the printer */
#define    LIMIT   100	   /* maximum number of iterations */
#define    MAXSIZE 4	   /* square of the maximum allowable magnitude of Z */

#define    READ    0	   /* file mode for open() */
#define    ESC_CHAR 27
#define    PRINTER 5	   /* os function for printer output */

int    count [MAXPTS];	   /* This is the array which holds the iteration */
			   /* counts for each point of the area to be viewed. */
			   /* This array will be filled up then sent to the */
			   /* printer as a complete line of bit images (960) */
			   /* each time a line is to be printed. */

char   line [MAXCOL];	   /* The line to be sent to the printer is assembled */
			   /* into this array one byte at a time. */

char   colors [LIMIT];	   /* This array will be used to hold the color */
			   /* distribution to be used for the plot. The */
			   /* values are stored externally in a text file */
			   /* then read in, converted to numeric values and */
			   /* stored in this array. Each iteration count from */
			   /* 0 to LIMIT must be represented by a number in */
			   /* the text file, and the numbers must be separated */
			   /* by a space or on a new line */

int    _rax,_rbx,_rcx,_rdx;/* register storage */
int    row, column;	   /* cursor row and column on the text screen */
int    grow, gcolumn;	   /* current row and column on the graphics screen */
FILE   picture; 	   /* the picture file descriptor */
FILE   colordat;	   /* color data file */

char   colname[13];	   /* name of color data file */
char   numstr[6];	   /* for reading in color file data */
char   time[9]; 	   /* time string buffer */
			   /* printer setup strings */
char   grafsp[] = { ESC_CHAR, 'A', 8, '\0' };
char   grafmode[] = { ESC_CHAR, 'L', '\0' };
char   preset[] = { ESC_CHAR, '@', '\0' };

double Creal;		   /* real part of number in center of area */
double Cimag;		   /* imaginary part of number in center of area */
double Old_Creal;	   /* save for restoring Creal */
double Old_Cimag;	   /* save for restoring Cimag */
double Width;		   /* width of area to be plotted */
double Height;		   /* Height of area to be plotted */
double dX, dY;		   /* delta increments for X and Y */
double Zreal;		   /* real component of Z */
double Zimag;		   /* imaginary component of Z */
double Size;		   /* magnitude of vector Z */

/*----- Function declarations ----------------------------------------------*/

char   next_ch(), chk_ch();
void   cls(), locate(), new_pic(), old_pic(), get_time(),
       graphics_on(), graphics_off(), pallette();
int    iterate(), save(), get_vmode();

/*--------------------------------------------------------------------------*/

main(argc,argv)
int    argc;
char   *argv[];
{
   int	   i,j;
   char    choice, *nextch;

   if ( argc < 2 )
       {
       puts("\nEnter the name of the color data file ==> ");
       gets(colname);
       }
   else
       strcpy ( colname, argv[1] );

   if ( ( colordat = fopen ( colname, "r" ) ) == NULL )
       {
       printf ( "\nCan't find the file %s\nProgram terminated.\n", colname);
       exit(1);
       }
   for ( i = 0; i < LIMIT; i++ )
       {
       fscanf( colordat, "%5s", numstr);
       colors [i] = atoi ( numstr );
       }

       cls();
       locate (0,0);
       puts("      MandelZoom -- by Brian Irvine\n");
       nextch = grafsp;
       while ( *nextch )
	   _os ( PRINTER, *nextch++ );
       new_pic();
       nextch = preset;
       while ( *nextch )
	   _os ( PRINTER, *nextch++ );

}


/*----- Start a new picture ------------------------------------------------*/

void   new_pic()
{
   int i,j, points;
   char    dummy;

   /* Get file name and parameters */

   puts("\nX coordinate (real part) of center => ");
   scanf( "%F", &Creal );
   puts("\nY coordinate (imaginary part) of center => ");
   scanf( "%F", &Cimag );
   puts("\nWidth of area to plot => ");
   scanf( "%F", &Width );

   /* calculate the X and Y increments */

   Height = Width * ASPECT;
   dX = Width / (double) (MAXCOL);
   dY = Height / (double) (MAXROW * 8);

   /* find the lower-left corner of the picture */

   Creal -= Width/2;
   Old_Creal = Creal;
   Cimag += Height/2;
   Old_Cimag = Cimag;

   /* open the file */

   cls();
   locate(0,0);
   puts("MandelZoom - by Brian Irvine\n");
   puts("\nCreating picture");
   puts("\nThe region being calculated is:\n");
   printf("     From %1.5F, %1.5F i\n",Creal, Cimag);
   printf("     To   %1.5F, %1.5F i\n",Creal+Width, Cimag-Height);
   get_time();
   printf("\nBegun at %s", time);
   puts("\n\nCurrently working on row ");

   /* start calculating */

   points = 0;
   for ( grow = 0; grow <= MAXROW * 8; grow++ )
       {
       locate ( 9,25 );
       printf ("%-3d", grow);
       for ( gcolumn = 0; gcolumn < MAXCOL; gcolumn++ )
	   {
	   count[points++] = iterate();
	   if ( points == MAXPTS )
	       {
	       print ( points );
	       points = 0;
	       }
	   Creal = Old_Creal + ( dX * gcolumn );
	   }
       Creal = Old_Creal;
       Cimag = Old_Cimag - ( dY * grow );
       }

   get_time();
   printf("\nFinished at: %s", time );
}



/*----- Calculate ----------------------------------------------------------*/
/*									    */
/*     Repeatedly calculate Z = Z + C until the iteration count reaches    */
/*     the maximum or until the magnitude of Z exceeds a threshold value.   */
/*     Returns the number of iterations.				    */
/*									    */
/*--------------------------------------------------------------------------*/


int    iterate()
{
   int	   count;
   char    ch;
   double  RealZ;

   Zreal = Zimag = 0;
   for ( count = 1; count <= LIMIT; count++ )
       {
       if ( (ch = chk_ch()) == ESC_CHAR )
	   {
	   close (picture);
	   exit(0);
	   }
       RealZ = Zreal;
       Zreal = RealZ * RealZ - Zimag * Zimag + Creal;
       Zimag = 2.0 * RealZ * Zimag + Cimag;
       Size = Zreal * Zreal + Zimag * Zimag;
       if ( Size > MAXSIZE )
	   return ( count );
       }
   return (LIMIT);

}


/*----- Send the next complete line to the printer -------------------------*/
/*									    */
/*     This routine reads 8 bits of data from each of 8 different bytes     */
/*     and assembles them into a byte to be sent to the printer. See the    */
/*     Epson manual for a description of how to perform this operation.     */
/*     The operation is performed for each value in the count[] array,	    */
/*     creating an array of 960 bytes which are sent to the printer as a    */
/*     complete line of bitmode values. 				    */
/*									    */
/*--------------------------------------------------------------------------*/

int    print( number )
int    number;
{

   int	   i, bit;
   char    byte, color, *nextch;

   /* assemble the line to be printed */

   for ( i = 0; i < MAXCOL; i++ )
       {
       for ( bit = 0; bit < 8; bit++ )
	   {
	   color = colors [count [( bit * MAXCOL ) + i] ] % 1;
	   byte <<= 1;
	   byte += color;
	   }
       line [i] = byte;
       }

   /* set printer up to print 960 bytes in bitmode */

   nextch = grafmode;
   while ( *nextch )
       _os ( PRINTER, *nextch++ );
   _os ( PRINTER, MAXCOL % 0x100 );
   _os ( PRINTER, MAXCOL / 0x100 );

   /* send the data to the printer */

   for ( i = 0; i < MAXCOL; i++ )
       _os ( PRINTER, line[i] );
   _os ( PRINTER, '\n' );

}


/****************************************************************************/
/*     Functions specific to DeSmet compiler				    */


/*----- Clear the screen ---------------------------------------------------*/

void   cls()
{
   scr_clr();

}


/*----- Set cursor position ------------------------------------------------*/

void   locate ( row, col )
int    row, col;
{
   scr_rowcol( row, col );

}


/*----- Wait for next character from keyboard ------------------------------*/

char   next_ch()
{
   char    ch;

   ch = scr_ci();
   return ch;
}


/*----- Check for character and return with it if present ------------------*/

char   chk_ch()
{
   char    ch;
   ch = scr_csts();
   return ch;

}


/*----- Get the time into the string time[] --------------------------------*/

void   get_time()
{
   times(time);
}
